//modified by Jing Yang after 7/6/2001
#ifndef CInterRingDisplay_H
#define CInterRingDisplay_H

#include <xmdv/CDisplay.h>
#include <xmdv/CFlatDisplayInformation.h>
#include <xmdv/InterRing/CInterRingClusterTree.h>
#include <xmdv/InterRing/CInterRingCluster.h>

enum MOUSE_MODE {
    INACTIVE,
    LEFT_DISTORTION_HANDLE,
    RIGHT_DISTORTION_HANDLE,
    MIDDLE_DISTORTION,
};

enum ACTION_MODE {
    CIRCULAR_DISTORTION,
    CIRCULAR_DISTORTION_PIN,
    RADIAL_DISTORTION,
    ROTATION,
    ROLLUP_DRILLDOWN,
    MODIFICATION,
    SELECTION,
    NONE_ACTION
};

enum SELECTION_MODE {
    ENTRIES_MODE,
    RADIUS_MODE
};

enum BRUSHING_MODE {
    COVER_ALL,
    FILTER_OUT
};

class CInterRingDisplay: public CDisplay
{
public:
    CInterRingClusterTree *tree;
	ARR<double> radius;
	Vec2 center; 
	double circle_offset;
    ACTION_MODE action_mode;
    SELECTION_MODE selection_mode;
    BRUSHING_MODE brushing_mode;

    bool circular_distortion_feedback;
    bool rollup_drilldown_feedback;
    bool modification_feedback;
    bool circular_distortion_with_pin;
    bool show_selected_name;

    CInterRingCluster* current_selection_node;
protected:
	CInterRingCluster* current_node;
    CInterRingCluster* prev_node;
	bool start_node;
	bool end_node;
	bool in_node;// if mouse is near the start of, the end of, or in the current_node, start>end>in

	MOUSE_MODE near_mouse_mode;
	MOUSE_MODE mouse_mode;

    //used for circular distortion with pin, record the pinned nodes
    CLinkList *pin_node_list;
    //used for radial distortion, record the pinned level
    int pin_level;

    double min_radius;

    //used for modificaton feedback
    double cursor_x;
    double cursor_y;

public:
	CInterRingDisplay();
	~CInterRingDisplay();
    
    //reset all parameters
    void Reset();
	
    //draw the canvas
    void Draw();
    void InputMouseMotion(int x, int y);
	void InputButtonPress(int button, int x, int y);
	void InputButtonRelease(int button, int x, int y);

    void SetActionMode(const char* argv);

    virtual CInterRingClusterTree* GetTree();

    void ResetRadius();
    void ResetSelection();
    void SBSelection(const char* argv_threshold);

protected:

    //draw a single cluster name
	void DrawSingleClusterName(CInterRingCluster *node);
	
	void InterRingCaption(int x, int y);

    void TraceDrawClusters(CInterRingCluster *node);

	void TraceDrawClusterOutlines(CInterRingCluster *node);

    void TraceDrawClusterNames(CInterRingCluster *node);
	
	Vec2 MapData(double radius, double angle);
	
	//draw a single cluster
	void DrawSingleCluster(CInterRingCluster *c);
	
	//draw a single cluster outline
	void DrawSingleClusterOutline(CInterRingCluster *node);

	//set tree related parameters used for the drawing;
	void SetTreeRelatedParameters();
	
	//free space used for drawing; to be rewritten by subclasses;
	void FreeSpace();
	
	void DrawCircles();

	CInterRingCluster* FindNode(double x, double y);
	
	CInterRingCluster* FindNode(double x, double y, bool &start, bool &end, bool &in);

    //find if (x, y) is near the end of or in a pinned node
    CInterRingCluster* FindPinnedNode(double x, double y, bool &start, bool &end, bool &in);

	void GetLevelAndAngleAccordingXY(double x, double y, int &level, double &angle);

	bool GetAngleAccordingXY(double x, double y, double &angle);

    void InputMouseMotion_CircularDistort(int x, int y);

    void InputMouseMotion_CircularDistortWithPin(int x, int y);

    void InputMouseMotion_RadialDistortion(int x, int y);

    void InputButtonPress_Selection(int button, int x, int y);
	
    void InputButtonRelease_CircularDistortWithPin(int button, int x, int y);

    void InputButtonRelease_RadialDistortion(int button, int x, int y);

    void DrawVisualFeedbacks();

    void TraceDrawDistortedFrame( CInterRingCluster *node );

    void DrawSingleClusterDistortedFrame(CInterRingCluster *node);
};

#endif

