
#if !defined(GU_GLOBLE_H)
#define GU_GLOBLE_H

#include <stdlib.h>
#include <string.h>
#include <iostream.h>

// -------------------- define constants used by GU -----------------------
#if defined(OKAY)
#undef OKAY
#endif
#define OKAY 1

#if defined(OOPS)
#undef OOPS
#endif
#define OOPS 0

#if defined(FALSE)
#undef FALSE
#endif
#define FALSE 0

#if defined(TRUE)
#undef TRUE
#endif
#define TRUE 1

#if defined(NOPE)
#undef NOPE
#endif
#define NOPE 0

#if defined(YEAH)
#undef YEAH
#endif
#define YEAH 1

#if defined(SUCCEED)
#undef SUCCEED
#endif
#define SUCCEED 1

#if defined(FAILED)
#undef FAILED
#endif
#define FAILED 0

#if defined(PI)
#undef PI
#endif
#define PI 3.1415926535897932384626
// ------------------ end of constants definition --------------------------


/**
 * Micro define to define a private data member in a class and give
 * two way of data access (non-const & const).
 * Example:
 *   gu_public_data( a, int )
 * which is equal to:
 *   private:
 *       int a_i;
 *   public:
 *       int& a() {return a_i;}
 *       const int& a() {return a_i;}
 **/
#define gu_public_data( data, Type ) \
private: \
	Type data ## _i; \
public: \
	Type& data() { return data ## _i; } \
        const Type& data() const { return data ## _i; }

// -----
#define gu_private_data( data, Type ) \
private: \
	Type data ## _i; \
	Type& data() { return data ## _i; } \
        const Type& data() const { return data ## _i; }

// -----
#define gu_public_array( data, Type, size ) \
private: \
	Type data ## _i ## [ ## size ## ]; \
public: \
	Type* data() { return data ## _i; } \
        const Type* const data() const { return data ## _i; }

// -----
#define gu_private_array( data, Type, size ) \
private: \
	Type data ## _i ## [ ## size ## ]; \
	Type* data() { return data ## _i; } \
        const Type* const data() const { return data ## _i; }


// -------------------------------------------------------------------------
// define a error output function, so the error messages of GU file can 
// be redirected, such as to a log file.
// -------------------------------------------------------------------------
inline void gu_error(char* a)
{
	cerr << a << endl;
}

// -------------------------------------------------------------------------
// It sucks to write a several lines to check the return value of new,
// why not make it looks clean in your program?
// -------------------------------------------------------------------------
template <class T>
inline int gu_new_single(T*& pt)
{
	pt = new T;
	if(pt)  return SUCCEED;
	gu_error("!gu_new:can't allocate memory.");
	return FAILED;
}


template <class T>
inline int gu_new_array(T*& pt, const int& size)
{
	pt = new T[size];
	if(pt)  return SUCCEED;
	gu_error("!gu_new:can't allocate memory.");
	return FAILED;
}

template <class T>
inline void gu_delete_single(T*& pt)
{
	delete pt;
}

template <class T>
inline void gu_delete_array(T*& pt)
{
	delete [] pt;
}
// ------------------ end of 'gu_new...' ---------------------------------


// -------------------------------------------------------------------------
// some small & frequently used functions.
// -------------------------------------------------------------------------
/*
inline double power2(const double& a)
{
	return a*a;
}

template <class T>
inline T max(const T& a, const T& b)
{
	return (a>b)? a:b;
}

template <class T>
inline T min(const T& a, const T& b)
{
	return (a<b)? a:b;
}
*/

 
// This swap can't be used to swap arrays/strings.
template <class T>
inline void swap(T& a, T& b)
{
	T temp = a;
	a = b;
	b = temp;
}

// max string length is 500.
inline void swap_string(char* a, char* b)
{
	char temp[500];
	strcpy(temp, a);
	strcpy(a, b);
	strcpy(b, temp);
}


// -------------------------------------------------------------------------
// Function used to see if two 'double' value is equal, the third input
// parameter is the tolerance, if no tolerance is input, use default 1E-10.
// -------------------------------------------------------------------------
inline int is_equal(const double& a, const double& b, const double& tolerance=1E-10)
{
	return ( (a-b)<tolerance && (b-a)<tolerance );
}

// -------------------------------------------------------------------------
// Function used to compare two string.
// -------------------------------------------------------------------------
inline int is_equal(const char* a, const char* b)
{
	return !strcmp(a, b);
}


#endif


