#include <xmdv/ogl.h>
#include <xmdv/CMixedDisplay.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/magnify.h>
#include <xmdv/brush_oper.h>
#include <xmdv/CFlatParcoord.h>
#include <xmdv/CHierParcoord.h>


CMixedDisplay::CMixedDisplay(CFlatDisplayInformation*  flat_info, CHierDisplayInformation* hier_info)
{
	this->flat_info = flat_info;
	this->hier_info = hier_info;
    this->curr_flat_display = new CFlatParcoord(flat_info);
    this->curr_hier_display = new CHierParcoord(hier_info);
    image = NULL;
}


CMixedDisplay::~CMixedDisplay()
{
}

void CMixedDisplay::SetCanvas(grCanvas *canvas)
{
    this->canvas = canvas;
    this->curr_flat_display->SetCanvas(canvas);
    this->curr_hier_display->SetCanvas(canvas);
}


//draw the canvas
void CMixedDisplay::Draw()
{
    if (!dataset) return;
    hierInitClusterTree();
    if ( !btree ) return;

    //set canvas background, clear canvas
	PrepareCanvas();

	glShadeModel(GL_SMOOTH);

	if ( curr_flat_display->TestDimensionality() ) //some display requires minimum dimensionality
	{
		//set parameters used for drawing
		curr_flat_display->SetParameters();
		curr_hier_display->SetParameters();

        int W, H;
        canvas->GetSize(&W, &H);
        if ( flat_info->redraw || W != image_W || H != image_H )
        {
		    //draw unbrushed data;
		    SetUnbrushedColorAndStyle();
		    DrawAllData();
            //draw axes
		    curr_flat_display->DrawAxes();

            canvas->Flush();
            SavePixels();

            flat_info->redraw = false;
        }
        else
        {
            LoadPixels();
        }
                                       

		
		//draw brushed data;
		DrawBrushedRegion();
    
        //draw brushes
		curr_flat_display->SetBrushesColorAndStyle();
		curr_flat_display->DrawBrushes();

        //free space used for drawing
		curr_hier_display->FreeSpace();
		curr_flat_display->FreeSpace();
	}
}


//set color and style for unbrushed data
//rewrite when necessary
void CMixedDisplay::SetUnbrushedColorAndStyle()
{
    RGBt c(interface_color[DATA]);
    glColor3f((float) c.R, (float) c.G, (float) c.B );
    canvas->SetForeground(interface_color[DATA], 0.2);
}

void CMixedDisplay::SavePixels()
{
    if ( image != NULL )
        delete image;

    int W, H;
    canvas->MakeCurrent();
    canvas->GetSize(&W, &H);
    image_W = W;
    image_H = H;

    image = new char[4*W*H];
//    image = new char[4000000];
//    Togl_PostRedisplay(canvas->GetWidget());
//    canvas->Flush();
//    canvas->MakeCurrent();
    glReadBuffer(GL_BACK);
    glReadPixels(0,0,W,H,GL_RGBA,GL_UNSIGNED_BYTE,image);
//    canvas->WriteImage( "test.ppm" );

}

void CMixedDisplay::LoadPixels()
{
    int W, H;
    canvas->MakeCurrent();
    canvas->GetSize(&W, &H);

//    gl.glClearColor(1, 1, 1, 1);
//    gl.glClear(GL_COLOR_BUFFER_BIT);
//    canvas->MakeCurrent();

//    for ( int i = 0; i < 4000000; i++)
//        image[i] = 'R';
    glDrawBuffer(GL_BACK);
    glDrawPixels(W,H,GL_RGBA,GL_UNSIGNED_BYTE,image);

}


//shiping
#include <xmdv/gu_link.h>
extern Link_List<RGBt> magenta, heatedobject;
//shiping

//draw all data
void CMixedDisplay::DrawAllData()
{
/*    //if MAGNIGY, should not draw unbrushed region.
    if ( MAGNIFY ) 
        return;
*/	
    int i;
    ARR<double> data;
    data.SetLen(dataset->dims);
    for (i=0; i<dataset->data_size; i++)
    {
        // First time through only paint regular data
        //
        if (dataset->mark[i] == okcdata::DISABLED) continue;
//        if (dataset->mark[i] != okcdata::REGULAR) continue;

        dataset->GetData(data, i);
        RGBt c;
        c = RGBt(interface_color[DATA]);

		glColor3f((float)c.R, (float)c.G, (float)c.B );
        canvas->SetForeground(c);
        curr_flat_display->DrawSingleData(data, i);
    }
}

//draw brushed data
void CMixedDisplay::DrawBrushedRegion()
{
   	//if both means and bands of the clusters need not to be shown, return;
	if ( ! ( hier_info->SHOW_MEANS || hier_info->SHOW_BANDS ) )
		return;
	
	//set current opacity
	double opacity = 1.0;
    if (hier_info->fade_factor<0) 
		opacity = 1+hier_info->fade_factor;
	curr_hier_display->SetCurrentOpacity( opacity );

    TraceDrawFSBBrushedClusters(btree->nodes[0]);
}

void CMixedDisplay::TraceDrawFSBBrushedClusters( Cluster node )
{
    if ( !node.contain_flat_brushed )
        return;

    if ( node.flat_brushed )
        curr_hier_display->DrawSingleCluster1(node, true);
    else
    {
        for (int j=0; j<node.num_children; j++) 
            TraceDrawFSBBrushedClusters(btree->nodes[node.child[j]]);
    }
}

void CMixedDisplay::InputKeyPress(int key)
{
    curr_flat_display->InputKeyPress(key);
}


void CMixedDisplay::InputKeyRelease(int key)
{
    curr_flat_display->InputKeyRelease(key);
}


void CMixedDisplay::InputMouseMotion(int x, int y)
{
    curr_flat_display->InputMouseMotion(x, y);
}


void CMixedDisplay::InputButtonPress(int button, int x, int y)
{
    curr_flat_display->InputButtonPress(button, x, y);
}


void CMixedDisplay::InputButtonRelease(int button, int x, int y)
{
    curr_flat_display->InputButtonRelease(button, x, y);
}



